# -*- coding: utf-8 -*-

# Copyright :copyright: 2019 by IBPort. All rights reserved.
# @Author: Neal Wong
# @Email: ibprnd@gmail.com

import os
import sys
import logging
import logging.handlers

from amazon_reports_downloader.models import init_db, Store



from amazon_reports_downloader.lib.google.sheet_api import SheetAPI
from amazon_reports_downloader import logger
from amazon_reports_downloader.bin.get_code import get_code
import time
import pytz

chicagoTz = pytz.timezone("America/Chicago") 
sheet_api = SheetAPI()

def get_qr_code_image_url_list():
    sheet_name = "code"
    sheet_id = "1b8wHMP05Na4ELcyP5Jn9syb9g6uMc8FkBTsckOCIz7w"
    worksheet = sheet_api.create_new_sheet_if_not_existed(sheet_id, sheet_name=sheet_name)

    headers = sheet_api.row_values(worksheet, 2)
    account_countries_column = headers.index("账号-卖场") + 1
    url_column = headers.index("qr_code") + 1
    if account_countries_column <= 0 or url_column <= 0:
        return None

    links = sheet_api.col_values(worksheet, url_column)
    account_countries = sheet_api.col_values(worksheet, account_countries_column)

    links_count = len(links)
    image_links = {}
    for i in range(len(account_countries)-1):
        if i < 2:
            continue

        if i < links_count:
            image_links[account_countries[i]] = links[i]
        else:
            image_links[account_countries[i]] = None


    return image_links

def get_stores():
    local_config_file_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "config_local.ini")
    config_path = local_config_file_path

    init_db(config_path)

    stores = []
    while True:
        try:
            stores = Store.select()
            break
        except peewee.OperationalError as e:
            logger.exception(e)

            time.sleep(3)
        except Exception as e:
            logger.exception(e)
            break
    return { store.account_country: store for store in stores}

def update_code():
    if sys.platform.startswith('win'):
        work_dir = 'C:\\AmazonReportDownloader'
    else:
        work_dir = os.path.join(os.path.expanduser('~'), '.AmazonReportDownloader')

    log_dir = os.path.join(work_dir, 'logs')
    if not os.path.isdir(log_dir):
        os.makedirs(log_dir)

    log_path = os.path.join(log_dir, 'disburse.log')
    level = logging.INFO
    max_bytes = 100 * 1024 ** 2
    fh = logging.handlers.RotatingFileHandler(
        log_path, maxBytes=max_bytes, backupCount=5)
    fh.setLevel(level)
    formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
    fh.setFormatter(formatter)
    logger.addHandler(fh)

    image_links = get_qr_code_image_url_list()
    stores = get_stores()

    if not image_links:
        return None

    for account_country, link in image_links.items():
        if not link:
            continue

        if account_country in stores:
            if stores[account_country].qr_code_image != link:
                stores[account_country].qr_code_image = link
                stores[account_country].save()
                get_code(account_country, True)
        else:
            store_data = {
                'account_country': account_country,
                'qr_code_image': link
            }
            Store.create(**store_data)
            get_code(account_country, True)

if __name__ == '__main__':
    update_code()
