# -*- coding: utf-8 -*-
import re
import json
import traceback
import datetime
import pdb

import dateutil
import dateutil.parser

import asyncio
import json
import dateutil.parser
from pyppeteer import launch
from pyppeteer.errors import TimeoutError, ElementHandleError
from amazon_reports_downloader.utils import extract_balance_amount


class DisbursementsManager(object):
    def __init__(self, page, marketplace, rate=1):
        self.page = page
        self.marketplace = marketplace.lower()
        self.rate = rate

    async def get_disbursements(self):
        disbursements_selector = '.financial-event-group-search-results'
        disbursements = []

        while True:
            try:
                disbursements_elem = await self.page.waitForSelector(disbursements_selector, timeout=60000)
                disbursement_elems = await self.page.querySelectorAll(disbursements_selector)

                for disbursement_elem in disbursement_elems:
                    disbursement = {'status': 'active'}

                    timerange_elem = await disbursement_elem.querySelector('.financial-event-group-metadata .group-timerange')
                    timerange_text = await self.page.evaluate('(node) => node.innerText.trim()', timerange_elem)

                    if 'Present' in timerange_text:
                        continue

                    if (await self.find_no_disbursement()):
                        return []

                    disbursement['timerange'] = timerange_text

                    workflow_elem = await disbursement_elem.querySelector('.fund-transfer-component .fund-transfer-box .fund-transfer-workflow kat-workflowtracker')
                    workflow_steps = None
                    if workflow_elem:
                        workflow_steps = await self.page.evaluate('(node) => JSON.parse(node.getAttribute("steps"))', workflow_elem)

                    if workflow_steps:
                        for item in workflow_steps:
                            label_lower = item['label'].lower()

                            if 'scheduled payout' in label_lower or 'scheduled disbursement' in label_lower:
                                scheduled_payout_time_s = label_lower.replace('scheduled payout', '').replace('scheduled disbursement', '').replace('\n', ' ').strip()
                                scheduled_payout_time = dateutil.parser.parse(scheduled_payout_time_s)
                                disbursement['scheduled_payout_time'] = scheduled_payout_time
                                disbursement['inaccurate_payout_time'] = scheduled_payout_time.hour == 0 and scheduled_payout_time.minute == 0 and scheduled_payout_time.second == 0
                                break
                    else:
                        # Already disbursed but Amazon didn't update payout information yet
                        # Just record a probably time to keep consistent
                        disbursement['scheduled_payout_time'] = (datetime.datetime.now() - datetime.timedelta(minutes=20))
                        disbursement['status'] = 'inactive'
                        continue

                    payout_status_elem = await disbursement_elem.querySelector('.payout-status .status kat-statusindicator')
                    payout_status = await self.page.evaluate('(node) => node.getAttribute("label")', payout_status_elem)
                    disbursement['payout_status'] = payout_status.lower()

                    payout_amount_elem = await disbursement_elem.querySelector('.payout-status .payout')
                    payout_amount = await self.page.evaluate('(node) => node.innerText.trim()', payout_amount_elem)
                    disbursement['payout_amount'] = round(extract_balance_amount(self.marketplace, payout_amount) / self.rate, 2)

                    disbursements.append(disbursement)

                break
            except ElementHandleError as e:
                print(f"ElementHandleError: {e}")
            except TimeoutError:
                # Handle timeout
                pass
            except Exception as e:
                print(e)
                await self.page.reload()

        return disbursements

    async def find_no_disbursement(self):
        try:
            await self.page.waitForSelector(
                "#content-feg-1-1 > div > div.fund-transfer-component.financial-event-group-column.lg-column > div > div > kat-alert"
            )

            text_content = await self.page.evaluate('''
                () => {
                    const element = document.querySelector("#content-feg-1-1 > div > div.fund-transfer-component.financial-event-group-column.lg-column > div > div > kat-alert");
                    const shadowRoot = element.shadowRoot;
                    const slot = shadowRoot.querySelector("div.content > div.description > slot");
                    return slot.textContent.trim();
                }
            ''')

            return text_content.find('No disbursement') > -1
        except Exception as e:
            pass

        return False


    async def get_present_settlement(self):
        disbursements_selector = '.financial-event-group-search-results'
        present_disbursement = None

        while True:
            try:
                disbursement_elems = await self.page.querySelectorAll(disbursements_selector)

                for disbursement_elem in disbursement_elems:
                    disbursement = {'status': 'active'}

                    timerange_elem = await disbursement_elem.querySelector('.financial-event-group-metadata .group-timerange')
                    timerange_text = await self.page.evaluate('(node) => node.innerText.trim()', timerange_elem)

                    if 'Present' not in timerange_text:
                        continue

                    if (await self.find_no_disbursement()):
                        return []

                    disbursement['timerange'] = timerange_text

                    workflow_elem = await disbursement_elem.querySelector('.fund-transfer-component .fund-transfer-box .fund-transfer-workflow kat-workflowtracker')
                    workflow_steps = None
                    if workflow_elem:
                        workflow_steps = await self.page.evaluate('(node) => JSON.parse(node.getAttribute("steps"))', workflow_elem)

                    if workflow_steps:
                        for item in workflow_steps:
                            label_lower = item['label'].lower()

                            if 'scheduled payout' in label_lower:
                                scheduled_payout_time_s = label_lower.replace('scheduled payout', '').replace('\n', ' ').strip()
                                scheduled_payout_time = dateutil.parser.parse(scheduled_payout_time_s)
                                disbursement['scheduled_payout_time'] = scheduled_payout_time
                                disbursement['inaccurate_payout_time'] = scheduled_payout_time.hour == 0 and scheduled_payout_time.minute == 0 and scheduled_payout_time.second == 0
                                break
                    else:
                        # Already disbursed but Amazon didn't update payout information yet
                        # Just record a probably time to keep consistent
                        disbursement['scheduled_payout_time'] = (datetime.datetime.now() - datetime.timedelta(minutes=20))
                        disbursement['status'] = 'inactive'
                        continue

                    payout_status_elem = await disbursement_elem.querySelector('.payout-status .status kat-statusindicator')
                    payout_status = await self.page.evaluate('(node) => node.getAttribute("label")', payout_status_elem)
                    disbursement['payout_status'] = payout_status.lower()
                    payout_amount_elem = await disbursement_elem.querySelector('.payout-status .payout')
                    payout_amount = await self.page.evaluate('(node) => node.innerText.trim()', payout_amount_elem)
                    disbursement['payout_amount'] = extract_balance_amount(self.marketplace, payout_amount)

                    present_disbursement = disbursement

                break
            except TimeoutError:
                # Handle timeout
                pass
            except Exception as e:
                print(e)
                await self.page.reload()

        return present_disbursement
