# -*- coding: utf-8 -*-

# Copyright © 2018 by IBPort. All rights reserved.
# @Author: Neal Wong
# @Email: ibprnd@gmail.com

import asyncio
import requests
import time
import logging
import os
import re
from pyppeteer.errors import PageError

# import cv2
import pyotp

from amazon_reports_downloader import MARKETPLACE_MAPPING, MARKETPLACE_MAPPING_V2
from amazon_reports_downloader import logger


class SellerLoginHelper(object):
    
    def __init__(self, page, email, password, marketplace, code, sheet_api=None, sheet_id=None):
        self.page = page
        self.email = email
        self.password = password
        self.marketplace = marketplace.lower()
        self.sheet_api = sheet_api
        self.code = code
        self.marketplace = marketplace
        self.sheet_id = sheet_id

    async def is_login_required(self):
        try:
            await asyncio.gather(
                self.page.waitForNavigation({'waitUntil': 'domcontentloaded', 'timeout': 5000})
            )
        except:
              pass
        url = self.page.url
        return url.find('/ap/signin') != -1 or url.find('/ap/mfa') != -1

    async def login(self):
        try:
            claimed_email_elem = await self.page.querySelector('#ap-claim')

            if claimed_email_elem:
                claimed_email = await self.page.evaluate('(elem) => elem.value', claimed_email_elem)
                if claimed_email.lower() != self.email.lower():
                    add_account_elem = await self.page.querySelector('#cvf-account-switcher-add-accounts-link')
                    if add_account_elem:
                        add_account_link = await self.page.evaluate('(elem) => elem.href', add_account_elem)
                        if add_account_link:
                            await self.page.goto(add_account_link)

        except Exception as e:
            print(f"Error: {e}")

        try:
            email_elem = await self.page.querySelector('#ap_email')
            await self.page.evaluate("(elem) => elem.value=''", email_elem)
            await email_elem.type(self.email)

            try:
                continue_elem = await self.page.querySelector('#continue')
                if continue_elem:
                    await continue_elem.click()
            except Exception as e:
                pass

        except Exception as e:
            print(f"Error: {e}")

        time.sleep(7)

        try:
            try:
                remember_elem = await self.page.querySelector('input[name="rememberMe"]')
                if remember_elem and not await self.page.evaluate('(elem) => elem.checked', remember_elem):
                    await remember_elem.click()
            except Exception:
                pass

            password_elem = await self.page.querySelector('#ap_password')

            await self.page.evaluate('(elem) => elem.value = ""', password_elem)
            await password_elem.type(self.password)
            await asyncio.sleep(3)
            await password_elem.press('Enter')
            await self.page.waitForNavigation()
        except Exception as e:
            print(f"Error: {e}")

    async def mfa_exist(self):
        try:
            mfa_form_xpath = '//form[@id="auth-mfa-form"] | //form[@id="verification-code-form"]'
            await self.page.waitForXPath(mfa_form_xpath, {'visible': True, 'timeout': 7000})
            print("mfa exist")
            return True
        except Exception as e:
            print("mfa dose not exist")
        return False

    def generate_otp_from_uri(self, totp_uri):
        if not totp_uri:
            return None

        retry_left_count = 5
        while retry_left_count > 0:
            try:
                totp = pyotp.parse_uri(totp_uri)
                current_otp = totp.now()
                return current_otp
            except pyotp.utils.UtilsException as e:
                print(f"Error parsing URI: {e}")
            except pyotp.otp.TotpError as e:
                print(f"Error generating TOTP: {e}")
            except Exception as e:
                print(f"Unexpected error: {e}")
            finally:
                retry_left_count -= 1

    async def send_otp_code(self, code):
        if not code:
            return None

        try:
            email_elem = await self.page.querySelector('input#auth-mfa-otpcode, input#input#auth-mfa-otpcode')
            await self.page.evaluate("(elem) => elem.value=''", email_elem)
            await email_elem.type(code)

            try:
                no_need_otp_checkbox = await self.page.querySelector('input#auth-mfa-remember-device')
                if no_need_otp_checkbox:
                    await no_need_otp_checkbox.click()
            except PageError:
                pass

            try:
                submit_elem = await self.page.querySelector('input#auth-signin-button')
                if submit_elem:
                    await submit_elem.click()
            except PageError:
                pass

            try:
                submit_elem = await self.page.querySelector('span#cvf-submit-otp-button input')
                if submit_elem:
                    await submit_elem.click()
            except PageError:
                pass

        except Exception as e:
            print(f"Error: {e}")


    # def extract_otp_code_uri_from_qr(self, image_path):
    #     if not image_path:
    #         return
    #
    #     image = cv2.imread(image_path)
    #     qr_code_decoder = cv2.QRCodeDetector()
    #     retval, decoded_info, points, straight_qrcode = qr_code_decoder.detectAndDecodeMulti(image)
    #     if not retval:
    #         return None
    #
    #     for info in decoded_info:
    #         if 'otpauth' in info:
    #             return info

    def get_qr_code_image_url(self):
        sheet_name = "code"
        worksheet = self.sheet_api.create_new_sheet_if_not_existed(self.sheet_id, sheet_name=sheet_name)
        key = self.code + '-' + self.marketplace
        cell = self.sheet_api.find_by_key(worksheet, key)
        row = cell.row
        url_column = 13
        worksheet.cell(row, url_column).value
        return worksheet.cell(row, url_column).value

    def remove_qc_code_image(self, save_path):
        try:
            os.remove(save_path)
            print(f"File {save_path} deleted successfully.")
        except FileNotFoundError:
            print(f"File {save_path} not found.")
        except Exception as e:
            print(f"Error deleting file: {e}")

    def change_image_url(self, image_url):
        match = re.search(r'/file/d/([a-zA-Z0-9_-]+)', image_url)
        if match:
            file_id = match.group(1)
            return f'https://drive.google.com/uc?id={file_id}&export=view&authuser=0'
        else:
            return None

    def get_image_type(self, response):
        type = None
        type_str = response.headers["content-type"]
        if type_str:
            type = type_str.split("/").pop()
        return type

    def get_qc_code_image(self):
        image_url = self.get_qr_code_image_url()
        response = requests.get(image_url, stream=True)
        if response.headers["content-type"].find("text/html") > -1:
            image_url = self.change_image_url(image_url)

        if not image_url:
            return

        # self.remove_qc_code_image(save_path)
        try:

            response = requests.get(image_url, stream=True)
            type = self.get_image_type(response)
            save_path = os.path.join(os.path.expanduser('~'), '.amazon_seller_management', f'{self.code}.{type}')
            if response.status_code == 200:
                save_dir = os.path.dirname(save_path)
                
                if not os.path.exists(save_dir):
                    os.makedirs(save_dir)

                with open(save_path, 'wb') as file:
                    for chunk in response.iter_content(chunk_size=128):
                        file.write(chunk)
                
                return save_path
            else:
                print(f"Failed to download image. Status code: {response.status_code}")

        except Exception as e:
            print(f"Error downloading image: {e}")

    async def send_otp(self):
        try:
            send_otp_button_selector = 'input#auth-send-code'
            send_otp_button = await self.page.querySelector(send_otp_button_selector)
            if send_otp_button:
                await send_otp_button.click()
                try:
                    await asyncio.gather(
                        self.page.waitForNavigation({'waitUntil': 'domcontentloaded', 'timeout': 5000})
                    )
                except:
                      pass
        except Exception as e:
            pass
        pass

    def get_code_from_gideon(self):
        url = "https://gideonwarriors.com/api/opt_code"

        payload = {
            'account_code': self.code + '-' + self.marketplace,
            'secret': 'vine153!!'
        }
        retry_left_count = 3
        while retry_left_count > 0:
            response = requests.post(url, data=payload)
            if response.status_code == 200:
                res = response.json()
                code = res.get("code") if res else None
                if code and code.isdigit() and len(code) == 6:
                    return code
                else:
                    break
            else:
                print("请求失败:", response.status_code)
                time.sleep(2)
                retry_left_count -= 1

    async def check_opt(self):
        try:
            is_mfa_exist = await self.mfa_exist()
            if is_mfa_exist:
                # image_path = self.get_qc_code_image()
                # code_uri = self.extract_otp_code_uri_from_qr(image_path)
                # code = self.generate_otp_from_uri(code_uri)
                code = self.get_code_from_gideon()
                await self.send_otp_code(code)
                # self.remove_qc_code_image(image_path)
        except Exception as e:
            pass

    async def pick_marketplace(self):
        try:
            button_xpath = '//div[@id="partner-switcher"]/button'
            await self.page.waitForXPath(button_xpath, {'visible': True, 'timeout': 7000})
            print("marketplace version is v2")
            return await self.pick_marketplace_v2()
        except Exception as e:
            print(e)
            print("marketplace version is v1")
        
        result = True

        marketplace_domain = MARKETPLACE_MAPPING.get(self.marketplace)['domain']
        picker_xpath = '//select[@id="sc-mkt-picker-switcher-select"]'
        target_xpath = picker_xpath + '//option[contains(text(), "{}")]'.format(marketplace_domain)

        try:
            picker_elem = await self.page.waitForXPath(picker_xpath, {'visible': True, 'timeout': 12000})
            cur_marketplace = await self.page.evaluate('(elem) => elem.options[elem.selectedIndex].text', picker_elem)

            store_elem = await self.page.waitForXPath('//*[@id="sc-mkt-picker-switcher-select"]/optgroup')
            store_name = await self.page.evaluate('(elem) => elem.label', store_elem)

            if cur_marketplace != marketplace_domain:
                try:
                    print('switch market place')
                    if marketplace_domain == 'www.amazon.com':
                        await picker_elem.select(f'{marketplace_domain} - {store_name}')
                    else:
                        await picker_elem.select(marketplace_domain)
                except Exception as e:
                    print(e)

            await self.page.waitForXPath(target_xpath, {'selected': True, 'timeout': 3000})
        except Exception as e:
            result = False

        return result

    async def pick_marketplace_v2(self):
        result = False
        current_marketplace_xpath = '//*[@id="partner-switcher"]/button'
        marketplace_elem = await self.page.waitForXPath(current_marketplace_xpath, {'visible': True, 'timeout': 4000})
        marketplace = await self.page.evaluate('(elem) => elem.textContent', marketplace_elem)
        logger.info(marketplace)

        marketplace_county = MARKETPLACE_MAPPING_V2.get(self.marketplace)['country']
        
        button_xpath = '//div[@id="partner-switcher"]/button'
        marketplace_elem_xpath = f'//a[contains(text(), "{marketplace_county}")]'
        current_marketplace_xpath = '//*[@id="partner-switcher"]/button'

        for _ in range(10):
            try:
                button_elem = await self.page.waitForXPath(button_xpath, {'visible': True, 'timeout': 10000})
                await button_elem.click()
                marketplace_elem = await self.page.waitForXPath(marketplace_elem_xpath, {'visible': True, 'timeout': 4000})
                await marketplace_elem.click()
                marketplace_elem = await self.page.waitForXPath(current_marketplace_xpath, {'visible': True, 'timeout': 4000})
                marketplace = await self.page.evaluate('(elem) => elem.textContent', marketplace_elem)
                logger.info(marketplace)
                if marketplace_county in marketplace:
                    result = True
                break
            except PageError:
                pass
        
        return result
