# -*- coding: utf-8 -*-

import os
import sys
import re
import base64
import subprocess
import shutil

# -*- coding: utf-8 -*-
from logging.handlers import RotatingFileHandler
import logging



def is_encrypted(sku):
    return sku.find('-') == -1 and sku.find('_') == -1

def remove_none_ascii(s, encoding='utf-8'):
    if encoding is None:
        encoding = 'utf-8'

    text_type = str if sys.version_info[0] == 3 else unicode
    if isinstance(s, text_type):
        s = s.encode(encoding)

    return re.sub(br'[^\x00-\x7f]*', br'', s).decode(encoding)

# functions to detect/fix double-encoded UTF-8 strings
# Based on http://blogs.perl.org/users/chansen/2010/10/coping-with-double-encoded-utf-8.html
DOUBLE_ENCODED = re.compile("""
\xC3 (?: [\x82-\x9F] \xC2 [\x80-\xBF]                                    # U+0080 - U+07FF
       |  \xA0       \xC2 [\xA0-\xBF] \xC2 [\x80-\xBF]                   # U+0800 - U+0FFF
       | [\xA1-\xAC] \xC2 [\x80-\xBF] \xC2 [\x80-\xBF]                   # U+1000 - U+CFFF
       |  \xAD       \xC2 [\x80-\x9F] \xC2 [\x80-\xBF]                   # U+D000 - U+D7FF
       | [\xAE-\xAF] \xC2 [\x80-\xBF] \xC2 [\x80-\xBF]                   # U+E000 - U+FFFF
       |  \xB0       \xC2 [\x90-\xBF] \xC2 [\x80-\xBF] \xC2 [\x80-\xBF]  # U+010000 - U+03FFFF
       | [\xB1-\xB3] \xC2 [\x80-\xBF] \xC2 [\x80-\xBF] \xC2 [\x80-\xBF]  # U+040000 - U+0FFFFF
       |  \xB4       \xC2 [\x80-\x8F] \xC2 [\x80-\xBF] \xC2 [\x80-\xBF]  # U+100000 - U+10FFFF
       )
""", re.X)

def is_double_encoded(s):
    return DOUBLE_ENCODED.search(s) and True or False

def decode_double_encoded(m):
    s = m.group(0)
    s = re.sub(r'[\xC2-\xC3]', '', s)
    s = re.sub(r'\A(.)', lambda m: chr(0xC0 | (ord(m.group(1)) & 0x3F)), s)
    return s

def fix_double_encoded(s):
    if not is_double_encoded(s):
        return s
    return DOUBLE_ENCODED.sub(decode_double_encoded, s)

def get_platform_architecture():
    if sys.platform.startswith('linux') and sys.maxsize > 2 ** 32:
        platform = 'linux'
        architecture = '64'
    elif sys.platform == 'darwin':
        platform = 'mac'
        architecture = '64'
    elif sys.platform.startswith('win'):
        platform = 'win'
        architecture = '32'
    else:
        raise RuntimeError('Could not determine chromedriver download URL for this platform.')

    return platform, architecture

def get_chrome_path():
    """
    :return: the path of chrome installed on client
    """
    path = None
    platform, _ = get_platform_architecture()
    if platform == 'linux':
        path = get_linux_executable_path()
    elif platform == 'mac':
        path = '/Applications/Google Chrome.app/Contents/MacOS/Google Chrome'
    elif platform == 'win':
        process = subprocess.Popen(
            ['reg', 'query', 'HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\chrome.exe'],
            stdout=subprocess.PIPE, stderr=subprocess.DEVNULL, stdin=subprocess.DEVNULL
        )
        output = process.communicate()
        if output and output[0] and len(output[0]) > 0:
            path = os.path.join(output[0].decode('UTF-8').strip().split('REG_SZ')[-1].strip(), 'chrome.exe')
        else:
            process = subprocess.Popen(
                ['powershell', '-command', '$(Get-ItemProperty -Path Registry::HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\chrome.exe).Path'],
                stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE
            )
            path = os.path.join(process.communicate()[0].decode('UTF-8').strip(), 'chrome.exe')
    else:
        return

    return path

def get_linux_executable_path():
    """
    Look through a list of candidates for Google Chrome executables that might
    exist, and return the full path to first one that does. Raise a ValueError
    if none do.
    :return: the full path to a Chrome executable on the system
    """
    for executable in (
        "google-chrome",
        "google-chrome-stable",
        "google-chrome-beta",
        "google-chrome-dev",
        "chromium-browser",
        "chromium",
    ):
        path = shutil.which(executable)
        if path is not None:
            return path
    raise ValueError("No chrome executable found on PATH")

def get_platform_architecture():
    if sys.platform.startswith('linux') and sys.maxsize > 2 ** 32:
        platform = 'linux'
        architecture = '64'
    elif sys.platform == 'darwin':
        platform = 'mac'
        architecture = '64'
    elif sys.platform.startswith('win'):
        platform = 'win'
        architecture = '32'
    else:
        raise RuntimeError('Could not determine chromedriver download URL for this platform.')

    return platform, architecture


def get_chrome_paths():
    """
    :return: the path of chrome installed on client
    """
    paths = []
    platform, _ = get_platform_architecture()
    if platform == 'linux':
        paths = get_linux_executable_paths()
    elif platform == 'mac':
        paths = ['/Applications/Google Chrome.app/Contents/MacOS/Google Chrome']
    elif platform == 'win':
        process = subprocess.Popen(
            ['reg', 'query', 'HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\chrome.exe'],
            stdout=subprocess.PIPE, stderr=subprocess.DEVNULL, stdin=subprocess.DEVNULL
        )
        output = process.communicate()
        if output and output[0] and len(output[0]) > 0:
            paths = [os.path.join(output[0].decode('UTF-8').strip().split('REG_SZ')[-1].strip(), 'chrome.exe')]
        else:
            process = subprocess.Popen(
                ['powershell', '-command', '$(Get-ItemProperty -Path Registry::HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\chrome.exe).Path'],
                stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE
            )
            paths = [os.path.join(process.communicate()[0].decode('UTF-8').strip(), 'chrome.exe')]
    else:
        return

    return paths

def get_linux_executable_paths():
    """
    Look through a list of candidates for Google Chrome executables that might
    exist, and return the full path to first one that does. Raise a ValueError
    if none do.
    :return: the full path to a Chrome executable on the system
    """
    paths = []
    for executable in (
        "google-chrome",
        "google-chrome-stable",
        "google-chrome-beta",
        "google-chrome-dev",
        "chromium-browser",
        "chromium",
    ):
        path = shutil.which(executable)
        if path is not None:
            paths.append(path)
    if(paths):
        return paths
    raise ValueError("No chrome executable found on PATH")


def set_logger_handler(name,log_file_name,log_level=logging.DEBUG,backupCount=5,stdout=True):
    # logger.removeHandler(stream_handler)
    logger = logging.getLogger(name)
    logger.setLevel(log_level)

    work_dir = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
    logs_dir = os.path.join(work_dir, 'logs')
    if not os.path.isdir(logs_dir):
        os.makedirs(logs_dir)
    try:
        os.system("echo hu199005 | sudo -S chmod -R  777 " + logs_dir + '/*')
    except Exception as e:
        print(e)

    log_path = os.path.join(logs_dir, log_file_name)
    max_bytes = 200 * 1024 ** 2
    fh = RotatingFileHandler(
        log_path, maxBytes=max_bytes, backupCount=backupCount)
    fh.setLevel(log_level)
    formatter = logging.Formatter('%(asctime)s %(name)s [%(levelname)s]:%(message)s')
    fh.setFormatter(formatter)
    logger.addHandler(fh)
    if(stdout):
        handler = logging.StreamHandler(sys.stdout)
        handler.setLevel(logging.DEBUG)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        handler.setFormatter(formatter)
        logger.addHandler(handler)
    return logger
