import os
import datetime
import shutil
import asyncio
import time

from pyppeteer.errors import TimeoutError
from pyppeteer import launch

from amazon_reports_downloader.lib import logger
from amazon_reports_downloader.lib.misc import get_chrome_path

shared_work_directory = os.path.join(os.path.expanduser('~'), '.amazon_seller_management')

class PuppeteerUtils():
  def __init__(self, country='US'):
    self._browser = None
    self.program_start_time = datetime.datetime.now()
    data_dir = os.path.join(shared_work_directory, country)
    self.country = country
    chrome_path = get_chrome_path()
    if not chrome_path:
      raise RuntimeError('Chrome must be installed to run this program!')

    if not os.path.isdir(data_dir):
      os.makedirs(data_dir)

    self.launch_opts = {
      'headless': False,
      'ignoreHTTPSErrors': True,
      'executablePath': chrome_path,
      'ignoreDefaultArgs': True,
      'slowMo': 0.05,
      'devtools': False,
      'defaultViewport': None,
      'args': ['--start-maximized', '--user-data-dir={}'.format(data_dir)]
    }

  async def fetch_url_v2(self, url, width=1280, height=1024, timeout=20000):
    page = await (await self.browser()).newPage()
    # await page.setViewport({'width': width, 'height': height})

    max_retries = 3
    while max_retries >= 0:
      try:
        response = await page.goto(url, {'waitUntil': 'domcontentloaded', 'timeout': timeout})
        if response.status == 404:
          break
        if response.status == 200:
          break
        elif response.status >= 300 and response.status < 400:
          await asyncio.sleep(3)
          break
        elif response.status >= 400 and response.status < 500:
          break
        else:
          await asyncio.sleep(7)
          max_retries -= 1
      except TimeoutError:
        cookies = await page.cookies()
        for cookie in cookies:
          await page.deleteCookie(cookie)
        max_retries -= 1
        while page:
          try:
            await page.close()
            page = None

            break
          except:
            pass

        break
      except Exception as e:
        logger.exception(e)
        await asyncio.sleep(3)

    return page

  async def browser(self, force=False):
    launch_args = list(self.launch_opts.get('args', []))
    launch_opts = dict(self.launch_opts)
    launch_opts['args'] = launch_args

    if self._browser is None:
      launch_opts['args'] = list(set(launch_opts['args']))
      self._browser = await launch(launch_opts)

    return self._browser
