import os

from cmutils.config_loaders import IniConfigLoader
from peewee import *
import json
from playhouse.shortcuts import ReconnectMixin


database_proxy = Proxy()
cs_database_proxy = Proxy()


class ReconnectMySQLDatabase(ReconnectMixin, MySQLDatabase):
    pass


def init_db(config_path=None):
    config = IniConfigLoader(config_path)
    config.set_section('db')

    db_connection = ReconnectMySQLDatabase(
        host=config.get('host'),
        user=config.get('user'),
        password=config.get('password'),
        database=config.get('database')
    )
    database_proxy.initialize(db_connection)


def init_cs_db(config_path=None):
    config = IniConfigLoader(config_path)
    config.set_section('csdb')

    cs_db_connection = ReconnectMySQLDatabase(
        host=config.get('host'),
        user=config.get('user'),
        password=config.get('password'),
        database=config.get('database')
    )
    cs_database_proxy.initialize(cs_db_connection)


class BaseModel(Model):
    def __str__(self):
        r = {}
        for k in self._data.keys():
            try:
                r[k] = str(getattr(self, k))
            except:
                r[k] = json.dumps(getattr(self, k))
        return str(r)

    class Meta:
        database = database_proxy


class CsBaseModel(Model):
    # connect to another database
    def __str__(self):
        r = {}
        for k in self._data.keys():
            try:
                r[k] = str(getattr(self, k))
            except:
                r[k] = json.dumps(getattr(self, k))
        return str(r)

    class Meta:
        database = cs_database_proxy


class Store(BaseModel):
    account_country = CharField()
    qr_code_image = CharField()
    id = IntegerField()
    created_at = DateTimeField()
    updated_at = DateTimeField()

    @staticmethod
    def get_store(account_country):
        stores = Store.select().where(Store.account_country == account_country)
        if len(stores) == 0:
            return None
            # raise Exception("no store with name %s found" % name)
        for store in stores:
            return store


    class Meta:
        db_table = 'amazon_seller_accounts'


