# -*- coding: utf-8 -*-

# Copyright :copyright: 2019 by IBPort. All rights reserved.
# @Author: Neal Wong
# @Email: ibprnd@gmail.com

import re
import asyncio
from pyppeteer import launch
from pyppeteer.errors import PageError, TimeoutError
from pyppeteer.page import Page

from amazon_reports_downloader.utils import extract_balance_amount


class PaymentsManager(object):
    def __init__(self, page, marketplace):
        self.page = page
        self.marketplace = marketplace.lower()

    async def get_exchange_rate(self):
        while True:
            try:
                rate_elem = await self.page.waitFor(
                    '//kat-popover-content[@id="katal-id-4"]/span[1]/div/p[1]/span', {'timeout': 3000}
                )
                rate_str = await rate_elem.getProperty('innerHTML')
                rate_str = rate_str.asElement().get() if rate_str.asElement() else ''
                rate_str = rate_str.strip().split('=')[-1]
                rate_match = re.search(r'\d+\.\d+', rate_str)
                rate = float(rate_match.group()) if rate_match else None
                break
            except TimeoutError:
                rate = None
                break

        return rate

    async def get_instant_transfer_balance(self):
        balance = await self.get_instant_transfer_balance_v1()
        if balance is not None:
            return balance

        balance = await self.get_instant_transfer_balance_v2()

        return balance

    async def get_instant_transfer_balance_v1(self):
        summary_xpath = '//div[@class="pPaymentSummary"]' + \
            '[descendant::strong[contains(text(), "Balance available for transfer now")]]'

        balance = None
        while True:
            try:
                summary_elem = await self.page.waitFor(
                    summary_xpath, {'timeout': 1000}
                )
                balance_elem = await summary_elem.xpath(
                    './div[@class="pSummaryBlock"]/div[@class="pDetailLineValue"]'
                )
                balance_str = await self.page.evaluate('(element) => element.textContent', balance_elem[0])
                balance = extract_balance_amount(self.marketplace, balance_str)

                break
            except TimeoutError:
                break

        return balance

    async def get_instant_transfer_balance_v2(self):
        summary_xpath = '//kat-card[@class="linkable-multi-row-card"]' + \
            '[descendant::section[@class="available-balance-header"]]'
        balance_xpath = './div[@class="linkable-multi-row-card-rows-container"]/div[1]' + \
            '/div[@class="available-balance-row-with-children" or @class="available-balance-row"]' + \
            '/div[@class="available-currency-amount"]/span'

        balance = None
        while True:
            try:
                summary_elem = await self.page.waitFor(
                    summary_xpath, {'timeout': 12000}
                )
                balance_elem = await summary_elem.xpath(balance_xpath)
                balance_str = await self.page.evaluate('(element) => element.textContent', balance_elem[0])
                balance = extract_balance_amount(self.marketplace, balance_str)

                if '-' in balance_str:
                    balance *= -1

                break
            except TimeoutError:
                break

        return balance

    # async def get_instant_transfer_balance_v2(self):
    #     summary_xpath = '//kat-card[@class="linkable-multi-row-card"]' + \
    #         '[descendant::section[@class="available-balance-header"]]'
    #     balance_xpath = './div[@class="linkable-multi-row-card-rows-container"]/div[1]' + \
    #         '/div[@class="available-balance-row-with-children" or @class="available-balance-row"]' + \
    #         '/div[@class="available-currency-amount"]/span'
    #
    #     balance = None
    #     while True:
    #         try:
    #             summary_elem = await self.page.waitForSelector(summary_xpath, {'timeout': 12000})
    #             balance_elem = await summary_elem.xpath(balance_xpath)
    #             balance_str = await balance_elem[0].getProperty('textContent')
    #             balance_str = balance_str.asElement().get() if balance_str.asElement() else ''
    #             balance_str = balance_str.strip()
    #             balance = self.extract_balance_amount(balance_str)
    #
    #             if '-' in balance_str:
    #                 balance *= -1
    #
    #             break
    #         except Exception as e:
    #             print(f"An error occurred: {e}")
    #             break
    #
    #     return balance
    #
    def get_closing_balance(self):
        return (self.get_total_balance(), self.get_unavailable_balance())

    async def get_total_balance(self):
        total_balance = await self.get_total_balance_v1()
        if total_balance is not None:
            return total_balance

        return await self.get_total_balance_v2()

    async def get_total_balance_v1(self):
        total_balance = None

        while True:
            try:
                total_closing_balance_elem = await self.page.waitForXPath(
                    '//div[@class="pDetailLine"][descendant::a[@id="Total balance"]]', {'timeout': 1000})

                total_balance_elems = await total_closing_balance_elem.xpath('./div[@class="pDetailLineValue"]/span')
                total_balance_str = ''
                for total_balance_elem in total_balance_elems:
                    total_balance_str += await total_balance_elem.getProperty('textContent')
                total_balance_str = total_balance_str.strip()

                total_balance = extract_balance_amount(self.marketplace, total_balance_str)

                break
            except Exception as e:
                print(f"An error occurred: {e}")
                break

        return total_balance


    async def get_total_balance_v2(self):
        total_closing_balance_wrapper_xpath = '//kat-card[@class="linkable-multi-row-card"]' + \
            '[descendant::div[@class="total-balance-header"]]'
        total_balance_xpath = './div[@class="linkable-multi-row-card-rows-container"]/div[1]' + \
            '/div[@class="currency-amount" or @class="underline-link"]'

        total_balance = None
        while True:
            try:
                total_closing_balance_elem = await self.page.waitForXPath(
                    total_closing_balance_wrapper_xpath, {'timeout': 3000}
                )
                total_balance_elem = await total_closing_balance_elem.xpath(total_balance_xpath)
                if total_balance_elem is None:
                    break

                total_balance_str = await total_balance_elem[0].getProperty('textContent')
                total_balance_str = await total_balance_str.jsonValue()
                total_balance_str = total_balance_str.strip()
                total_balance = extract_balance_amount(self.marketplace, total_balance_str)
                if '-' in total_balance_str:
                    total_balance *= -1

                break
            except Exception as e:
                print(f"An error occurred: {e}")
                break

        return total_balance

    async def get_unavailable_balance(self):
        unavailable_balance = await self.get_unavailable_balance_v1()
        if unavailable_balance is not None:
            return unavailable_balance

        return await self.get_unavailable_balance_v2()

    async def get_unavailable_balance_v1(self):
        unavailable_balance = None

        while True:
            try:
                unavailable_closing_balance_elem = await self.page.waitForXPath(
                    '//div[@class="pDetailLine"][descendant::a[@id="Unavailable balance"]]',
                    {'timeout': 1000}
                )

                unavailable_balance_elems = await unavailable_closing_balance_elem.xpath('./div[@class="pDetailLineValue"]/span')
                unavailable_balance_str = ''
                for elem in unavailable_balance_elems:
                    text_content = await elem.getProperty('textContent')
                    text_content = text_content.asText().strip()
                    unavailable_balance_str += text_content

                unavailable_balance = extract_balance_amount(self.marketplace, unavailable_balance_str)

                break
            except Exception as e:
                print(f"An error occurred: {e}")
                break

        return unavailable_balance

    async def get_unavailable_balance_v2(self):
        unavailable_closing_balance_xpath = '//div[@class="top-level-breakdown-details"]' + \
            '/h4[text()="Account Level Reserve"]/following-sibling::div[contains(@class, "breakdown-amount")]'

        unavailable_balance = None

        while True:
            try:
                unavailable_balance_elem = await self.page.waitForXPath(unavailable_closing_balance_xpath, timeout=12000)

                unavailable_balance_str = await self.page.evaluate('(el) => el.textContent.trim()', unavailable_balance_elem)
                unavailable_balance = extract_balance_amount(self.marketplace, unavailable_balance_str)

                break
            except (TimeoutError):
                # Handle timeout or stale element reference
                break

        return unavailable_balance

    async def has_disburse_button_v1(self):
        has_disburse_button = True

        try:
            await self.page.waitForXPath('//*[@id="disburse_button_bottom_id"]', {'timeout': 1000})
        except Exception as e:
            has_disburse_button = None

        return has_disburse_button

    async def has_disburse_button_v2(self):
        has_disburse_button = True
        try:
            summary_elem = await self.page.waitForXPath(
                '//kat-card[@class="linkable-multi-row-card"][descendant::section[@class="available-balance-header"]]',
                {'timeout': 12000}
            )
            await summary_elem.waitForXPath(
                './div[@class="linkable-multi-row-card-rows-container"]/div[1]/div[@class="available-balance-row-with-children"]/div[@class="custom-child-available-balance"]/kat-button/button'
            )
        except Exception as e:
            has_disburse_button = None

        return has_disburse_button

    async def is_disburse_button_disabled(self, disburse_button):
        if not disburse_button:
            return None

        return bool(await disburse_button.getProperty('disabled'))

    async def get_disburse_button(self):
        disburse_button = await self.get_disburse_button_v1()
        if disburse_button:
            return disburse_button
        disburse_button = await self.get_disburse_button_v2()
        if disburse_button:
            return disburse_button
        return await self.get_disburse_button_v3()

    async def get_disburse_button_v1(self):
        disburse_button = None
        try:
            disburse_button = await self.page.waitForXPath('//*[@id="disburse_button_bottom_id"]', {'timeout': 1000})
            disburse_button = await self.page.waitForXPath('//*[@id="disburse_button_bottom_id"]', {'timeout': 1000})
        except Exception as e:
            pass

        return disburse_button

    async def get_disburse_button_v2(self):
        disburse_button = None
        try:
            summary_elem = await self.page.waitForXPath(
                '//kat-card[@class="linkable-multi-row-card"][descendant::section[@class="available-balance-header"]]',
                {'timeout': 12000}
            )
            disburse_button = await summary_elem.xpath(
                './div[@class="linkable-multi-row-card-rows-container"]/div[1]/div[@class="available-balance-row-with-children"]/div[@class="custom-child-available-balance"]/kat-button/button'
            )
        except Exception as e:
            pass

        return disburse_button

    async def get_disburse_button_v3(self):
        try:
            disabled = await self.page.evaluate("""() => document.querySelector("kat-card.linkable-multi-row-card div.linkable-multi-row-card-rows-container > div:first-child .available-balance-row-with-children .custom-child-available-balance kat-button").shadowRoot.querySelector("button").disabled""")
            if disabled:
                return 'disabled'

        except Exception as e:
            pass

        return True

    async def get_shadow_dom(self, shadow_host_elem):
        show_dom = await self.page.evaluate('(element) => element.shadowRoot', shadow_host_elem)
        return show_dom

    async def query_shadow_dom(self, shadow_host_elem, css_selector):
        return await self.page.evaluate('(host, selector) => host.shadowRoot.querySelector(selector)', shadow_host_elem, css_selector)

    async def query_shadow_dom_all(self, shadow_host_elem, css_selector):
        return await self.page.evaluate('(host, selector) => host.shadowRoot.querySelectorAll(selector)', shadow_host_elem, css_selector)

    async def trigger_disburse(self):
        try:
            await self.page.evaluate("""() => document.querySelector("kat-card.linkable-multi-row-card div.linkable-multi-row-card-rows-container > div:first-child .available-balance-row-with-children .custom-child-available-balance kat-button").shadowRoot.querySelector("button").click()""")
        except:
            return False

        return True

    async def has_disburse_button(self):
        has_disburse_button = await self.has_disburse_button_v1()
        if has_disburse_button is not None:
            return has_disburse_button

        return bool(await self.has_disburse_button_v2())


